/*
 * ImageFX Development Header File
 * Copyright  1991-1995 Nova Design, Inc.
 * Written by Thomas Krehbiel
 *
 * User Preferences.
 *
 */

#ifndef SCAN_PREFS_H


#ifndef SCAN_REQ_H
#include <scan/req.h>      /* for FileInfo struct */
#endif

/*
 * UserPrefs:
 *
 * Preferences settings that the user can change.  These settings
 * are saved to disk when requested.  These should all be considered
 * READ ONLY!
 */
struct UserPrefs {
   struct FileInfo   LoadPath;
   struct FileInfo   SavePath;
   struct FileInfo   RendPath;
#ifdef OLD
   struct FileInfo   BruLoadPath;
   struct FileInfo   BruSavePath;
   struct FileInfo   AlphaLoadPath;
   struct FileInfo   AlphaSavePath;
#else
   struct FileInfo   BrushPath;
   struct FileInfo   TexturePath;
   struct FileInfo   AlphaPath;
   struct FileInfo   fiReserved;
#define BruLoadPath     BrushPath      /* ancient compatibility, do not use */
#define BruSavePath     TexturePath
#define AlphaLoadPath   AlphaPath
#define AlphaSavePath   fiReserved
#endif
   struct FileInfo   ConvolvePath;
   struct FileInfo   TransformPath;
   struct FileInfo   ScannerMod;
   struct FileInfo   PreviewMod;
   struct FileInfo   RenderMod;
   struct FileInfo   VMemPath;
   struct FileInfo   ArexxPath;
   struct FileInfo   HookPath;
   struct FileInfo   FontPath;
   struct FileInfo   PalettePath;
   struct FileInfo   PrinterMod;
   struct FileInfo   SepPath;
   struct FileInfo   QuantizeMod;
   struct FileInfo   MaskPath;
   unsigned short    Palette[4];       /* Screen palette */
   char              SerDevice[32];    /* Serial device (OBSOLETE) */
   long              SerUnit;          /* Serial unit (OBSOLETE) */
   long              SerBaud;          /* Serial baud rate (OBSOLETE) */

   short             PanelDepth;       /* menu screen depth (2.0) */

   char              LoadReq,          /* 0=asl, 1=thumbnail (2.0) */
                     SaveReq,
                     MiscReq,
                     BufReq,
                     reqpad[2];

   ULONG             PanelMode;        /* panel screenmode (2.0) */
   ULONG             FreqMode;         /* thumbnail requester mode (2.0) */
   short             FreqDepth;        /* thumbnail requester depth (2.0) */
   short             reserved3;

   LONG              Version;          /* preferences version */
   WORD              TLE, TTE, TW, TH; /* thumbnail leftedge, topedge */
   long              reserved;         /* used to be parallel device & unit */

   long              Flags;            /* Various Flags (see below) */
   short             DitherLimit;      /* Dither limiting value (OBSOLETE) */
   short             VPageSize;        /* VMem page size (OBSOLETE) */
   short             VMaxPages;        /* VMem max pages in memory (OBSOLETE) */
   short             VMinThresh;       /* Minimum memory threshold (OBSOLETE) */
   char              CurMode;          /* Default starting panel mode (OBSOLETE) */
   char              Units;            /* Display units (see below) */
   char              ColorSpace;       /* Palette colorspace */
   char              Colors;           /* Palette colors - 1 */
   char              pr_DrawTool;      /* Drawing tool */
   char              FileReq;          /* File requester of choice */
   char              UseVMem;          /* When to use virtual memory */
   char              FontReq;          /* Font requester of choice */
   short             UndoLevels;       /* maximum undo levels */
   ULONG             VMaxRAM;          /* max RAM used by VMEM */
   ULONG             VMaxBuf;          /* max buffer size for VMEM */
   short             LE, TE, W, H;     /* EGS: left,top,width,height */
   char              ScreenMode[66];   /* EGS: screenmode */
   short             ScreenDepth;      /* EGS: screendepth */
   short             MLE, MTE, MW, MH; /* EGS: preview position */
   short             PLE, PTE, PW, PH; /* EGS: palette position */
   ULONG             EWindFlags;       /* EGS: window open flags */
   short             CLE, CTE;         /* EGS: coords window */
   ULONG             ValidFlags;       /* prefs struct validity flags */
};

#define PREF_VERSION    0x12340020     /* = Version 2.0 */

/* values for EWindFlags: */
#define EWF_TOOLBOX     0x00000001
#define EWF_PALETTE     0x00000002
#define EWF_PREVIEW     0x00000004

/*
 * Validity Flags
 */
#define VFF_THUMBSIZE      0x0000001L     /* TLE, TTE, TW, TH are valid */

/*
 * Values for the Flags field.
 */

#define PFF_CLOSEWB        0x0000002L     /* Close Workbench */
#define PFF_LACEPANEL      0x0000004L     /* Use interlaced menu panel */
#define PFF_NOUNDO         0x0000080L     /* Disable Undo */
#define PFF_COORDS         0x0000200L     /* Show coordinates */
#define PFF_ICONS          0x0000400L     /* Save icons with files */
#define PFF_KEEPPAL        0x0002000L     /* Keep palette on toolbox screen */
#define PFF_USEHEX         0x0004000L     /* Use hex for palette numbers */
#define PFF_METRIC         0x0008000L     /* Use metric instead of english */
#define PFF_POSTREND       0x0010000L     /* Call post render macro? */
#define PFF_NOCHECK        0x0020000L     /* Disable "are you sure?" requesters */
#define PFF_NOROLL         0x0040000L     /* Don't "roll" - snap instantly */
#define PFF_OPAQUE         0x0080000L     /* Use "opaque" menu panel */
#define PFF_CORRECTASPECT  0x0100000L     /* Use "correct aspect" preview! */
#define PFF_PRESSURE       0x0200000L     /* Enable pressure sensitivity? */
#define PFF_CUSTOMSCR      0x0400000L     /* Open on custom screen (EGS) */
#define PFF_SAVENAILS      0x0800000L     /* Save thumbnails with files (2.0) */
#define PFF_USENAILS       0x1000000L     /* Use thumbnails onscreen (2.0) */

/*
 * For ancient compatibility, do not use:
 */
#define PFF_ADGAMMA        0x000001L      /* Use coarse adjustment (obsolete) */
#define PFF_TODISK         0x000008L      /* Scan to disk (obsolete) */
#define PFF_HIRESPREV      0x000010L      /* Hires Preview (obsolete) */

#define PFF_COARSE         0x000001L      /* Use coarse adjustment (obsolete) */
#define PFF_AUTOPREV       0x000010L      /* AutoScale preview (obsolete) */
#define PFF_DITHERPREV     0x000020L      /* Dither Preview (obsolete) */
#define PFF_GREYPREV       0x000040L      /* Grey Preview (obsolete) */
#define PFF_AUTOBACK       0x000100L      /* Auto brush background (obsolete) */
#define PFF_ALLVMEM        0x000800L      /* Always use VMem buffers (obsolete) */
#define PFF_NOBUFFER       0x001000L      /* Create empty startup buffer (obsolete) */


/*
 * Units values:
 */
#define UN_PIXELS       (0)
#define UN_INCHES       (1)
#define UN_METRIC       (2)

/*
 * Defines for the Prefs.UseVMem field
 */
#define VMEM_NEVER      (0)               /* never use vmem (disabled) */
#define VMEM_ASK        (1)               /* ask before using */
#define VMEM_DONTASK    (2)               /* use when needed without asking */
#define VMEM_ALWAYS     (3)               /* always use whether needed or not */

#define SCAN_PREFS_H
#endif
